@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 our $DEFAULT_UA = "Perl + " . __PACKAGE__ . "/$VERSION";
 our $DEFAULT_MAXREDIR = 3;
@@ -28,7 +28,7 @@ use Socket qw( SOCK_STREAM );
 
 =head1 NAME
 
-C<Net::Async::HTTP> - Use HTTP with C<IO::Async>
+C<Net::Async::HTTP> - use HTTP with C<IO::Async>
 
 =head1 SYNOPSIS
 
@@ -113,6 +113,17 @@ be constructed that declares C<Net::Async::HTTP> and the version number.
 Optional. How many levels of redirection to follow. If not supplied, will
 default to 3. Give 0 to disable redirection entirely.
 
+=item proxy_host => STRING
+
+=item proxy_port => INT
+
+Optional. Default values to apply to each C<request> method.
+
+=item cookie_jar => HTTP::Cookies
+
+Optional. A reference to a L<HTTP::Cookies> object. Will be used to set
+cookies in requests and store them from responses.
+
 =back
 
 =cut
@@ -122,7 +133,7 @@ sub configure
    my $self = shift;
    my %params = @_;
 
-   foreach (qw( user_agent max_redirects )) {
+   foreach (qw( user_agent max_redirects proxy_host proxy_port cookie_jar )) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -245,6 +256,22 @@ Optional. The body content to use for C<POST> requests. If this is a plain
 scalar instead of an ARRAY ref, it will not be form encoded. In this case, a
 C<content_type> field must also be supplied to describe it.
 
+=item request_body => CODE or STRING
+
+Optional. Allows request body content to be generated by a callback, rather
+than being provided as part of the C<request> object. This can either be a
+C<CODE> reference to a generator function, or a plain string.
+
+As this is passed to the underlying L<IO::Async::Stream> C<write> method, the
+usual semantics apply here. If passed a C<CODE> reference, it will be called
+repeatedly whenever it's safe to write. The code should should return C<undef>
+to indicate completion.
+
+As with the C<content> parameter, the C<content_type> field should be
+specified explicitly in the request header, as should the content length
+(typically via the L<HTTP::Request> C<content_length> method). See also
+F<examples/PUT.pl>.
+
 =item content_type => STRING
 
 The type of non-form data C<content>.
@@ -317,9 +344,10 @@ sub do_request
    my $self = shift;
    my %args = @_;
 
-   my $on_response = $args{on_response} or
-      my $on_header = $args{on_header}  or croak "Expected 'on_response' or 'on_header' as CODE ref";
-   my $on_error    = $args{on_error}    or croak "Expected 'on_error' as a CODE ref";
+   my $on_response  = $args{on_response} or
+      my $on_header = $args{on_header}   or croak "Expected 'on_response' or 'on_header' as CODE ref";
+   my $on_error     = $args{on_error}    or croak "Expected 'on_error' as a CODE ref";
+   my $request_body = $args{request_body};
 
    my $max_redirects = defined $args{max_redirects} ? $args{max_redirects} : $self->{max_redirects};
 
@@ -327,10 +355,15 @@ sub do_request
    my $port;
    my $ssl;
 
+   my $request;
+
    my $on_header_redir = sub {
       my ( $response ) = @_;
 
       if( !$response->is_redirect or $max_redirects == 0 ) {
+         $response->request( $request );
+         $self->process_response( $response );
+
          return $on_header->( $response ) if $on_header;
          return sub {
             return $on_response->( $response ) unless @_;
@@ -373,8 +406,6 @@ sub do_request
       }
    };
 
-   my $request;
-
    if( $args{request} ) {
       $request = delete $args{request};
       ref $request and $request->isa( "HTTP::Request" ) or croak "Expected 'request' as a HTTP::Request reference";
@@ -426,7 +457,7 @@ sub do_request
       }
    }
 
-   $request->init_header( 'User-Agent' => $self->{user_agent} ) if length $self->{user_agent};
+   $self->prepare_request( $request );
 
    if( my $handle = $args{handle} ) { # INTERNAL UNDOCUMENTED
       my $transport = IO::Async::Stream->new( handle => $handle );
@@ -444,6 +475,7 @@ sub do_request
             my ( $conn ) = @_;
             $conn->request(
                request => $request,
+               request_body => $request_body,
                on_header => $on_header_redir,
                on_error  => $on_error,
             );
@@ -460,8 +492,8 @@ sub do_request
       }
 
       $self->get_connection(
-         host => $args{proxy_host} || $host,
-         port => $args{proxy_port} || $port,
+         host => $args{proxy_host} || $self->{proxy_host} || $host,
+         port => $args{proxy_port} || $self->{proxy_port} || $port,
          SSL  => $ssl,
 
          on_error => $on_error,
@@ -470,6 +502,7 @@ sub do_request
             my ( $conn ) = @_;
             $conn->request(
                request => $request,
+               request_body => $request_body,
                on_header => $on_header_redir,
                on_error  => $on_error,
             );
@@ -478,6 +511,43 @@ sub do_request
    }
 }
 
+=head1 SUBCLASS METHODS
+
+The following methods are intended as points for subclasses to override, to
+add extra functionallity.
+
+=cut
+
+=head2 $http->prepare_request( $request )
+
+Called just before the C<HTTP::Request> object is sent to the server.
+
+=cut
+
+sub prepare_request
+{
+   my $self = shift;
+   my ( $request ) = @_;
+
+   $request->init_header( 'User-Agent' => $self->{user_agent} ) if length $self->{user_agent};
+   $self->{cookie_jar}->add_cookie_header( $request ) if $self->{cookie_jar};
+}
+
+=head2 $http->process_response( $response )
+
+Called after a non-redirect C<HTTP::Response> has been received from a server.
+The originating request will be set in the object.
+
+=cut
+
+sub process_response
+{
+   my $self = shift;
+   my ( $response ) = @_;
+
+   $self->{cookie_jar}->extract_cookies( $response ) if $self->{cookie_jar};
+}
+
 # Keep perl happy; keep Britain tidy
 1;
 
